@@ -5,7 +5,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -105,7 +105,11 @@ gucharmap_chartable_accessible_ref_child (AtkObject *obj,
   GPtrArray *cells;
   guint n_cells, n;
 
+#if GTK_CHECK_VERSION (2, 21, 0)
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (obj));
+#else
   widget = GTK_ACCESSIBLE (obj)->widget;
+#endif
   if (widget == NULL)
     /* State is defunct */
     return NULL;
@@ -159,7 +163,11 @@ gucharmap_chartable_accessible_ref_at (AtkTable *table,
   AtkObject *child;
   gint index;
 
+#if GTK_CHECK_VERSION (2, 21, 0)
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (table));
+#else
   widget = GTK_ACCESSIBLE (table)->widget;
+#endif
   if (widget == NULL)
     /* State is defunct */
     return NULL;
@@ -186,7 +194,11 @@ gucharmap_chartable_accessible_ref_accessible_at_point (AtkComponent *component,
   gint x_pos, y_pos;
   gint row, col;
 
+#if GTK_CHECK_VERSION (2, 21, 0)
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (component));
+#else
   widget = GTK_ACCESSIBLE (component)->widget;
+#endif
   if (widget == NULL)
     /* State is defunct */
     return NULL;
@@ -244,7 +256,11 @@ gucharmap_chartable_accessible_ref_state_set (AtkObject *obj)
   GtkWidget *widget;
 
   state_set = ATK_OBJECT_CLASS (gucharmap_chartable_accessible_parent_class)->ref_state_set (obj);
+#if GTK_CHECK_VERSION (2, 21, 0)
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (obj));
+#else
   widget = GTK_ACCESSIBLE (obj)->widget;
+#endif
 
   if (widget != NULL)
     atk_state_set_add_state (state_set, ATK_STATE_MANAGES_DESCENDANTS);
@@ -259,7 +275,11 @@ gucharmap_chartable_accessible_get_n_children (AtkObject *obj)
 {
   GtkWidget *widget;
 
+#if GTK_CHECK_VERSION (2, 21, 0)
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (obj));
+#else
   widget = GTK_ACCESSIBLE (obj)->widget;
+#endif
   if (widget == NULL)
     /* State is defunct */
     return 0;
@@ -313,7 +333,11 @@ gucharmap_chartable_accessible_update_all_cells (AtkObject *obj)
   GPtrArray *cells;
   guint n_cells, n;
 
+#if GTK_CHECK_VERSION (2, 21, 0)
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (obj));
+#else
   widget = GTK_ACCESSIBLE (obj)->widget;
+#endif
   if (!widget)
     /* Widget is being deleted */
     return;
@@ -498,9 +522,15 @@ gucharmap_chartable_accessible_destroyed (GtkWidget *widget,
 static void
 gucharmap_chartable_accessible_connect_widget_destroyed (GtkAccessible *accessible)
 {
+#if GTK_CHECK_VERSION (2, 21, 0)
+  if (gtk_accessible_get_widget (accessible))
+    {
+      g_signal_connect_after (gtk_accessible_get_widget (accessible),
+#else
   if (accessible->widget)
     {
       g_signal_connect_after (accessible->widget,
+#endif
                               "destroy",
                               G_CALLBACK (gucharmap_chartable_accessible_destroyed),
                               accessible);
@@ -537,7 +567,11 @@ gucharmap_chartable_accessible_get_n_columns (AtkTable *table)
   GucharmapChartable *chartable;
   GucharmapChartablePrivate *chartable_priv;
 
+#if GTK_CHECK_VERSION (2, 21, 0)
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (table));
+#else
   widget = GTK_ACCESSIBLE (table)->widget;
+#endif
   if (widget == NULL)
     /* State is defunct */
     return 0;
@@ -556,7 +590,11 @@ gucharmap_chartable_accessible_get_column_extent_at (AtkTable       *table,
 {
   GtkWidget *widget;
 
+#if GTK_CHECK_VERSION (2, 21, 0)
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (table));
+#else
   widget = GTK_ACCESSIBLE (table)->widget;
+#endif
   if (widget == NULL)
     /* State is defunct */
     return 0;
@@ -573,7 +611,11 @@ gucharmap_chartable_accessible_get_n_rows (AtkTable *table)
   GucharmapChartablePrivate *chartable_priv;
   gint n_rows;
 
+#if GTK_CHECK_VERSION (2, 21, 0)
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (table));
+#else
   widget = GTK_ACCESSIBLE (table)->widget;
+#endif
   if (widget == NULL)
     /* State is defunct */
     return 0;
@@ -594,7 +636,11 @@ gucharmap_chartable_accessible_get_row_extent_at (AtkTable *table,
 {
   GtkWidget *widget;
 
+#if GTK_CHECK_VERSION (2, 21, 0)
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (table));
+#else
   widget = GTK_ACCESSIBLE (table)->widget;
+#endif
   if (widget == NULL)
     /* State is defunct */
     return 0;
@@ -612,7 +658,11 @@ gucharmap_chartable_accessible_get_index_at (AtkTable *table,
   GucharmapChartable *chartable;
   GucharmapChartablePrivate *chartable_priv;
 
+#if GTK_CHECK_VERSION (2, 21, 0)
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (table));
+#else
   widget = GTK_ACCESSIBLE (table)->widget;
+#endif
   if (widget == NULL)
     /* State is defunct */
     return -1;
@@ -632,7 +682,11 @@ gucharmap_chartable_accessible_get_column_at_index (AtkTable *table,
   GucharmapChartable *chartable;
   GucharmapChartablePrivate *chartable_priv;
 
+#if GTK_CHECK_VERSION (2, 21, 0)
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (table));
+#else
   widget = GTK_ACCESSIBLE (table)->widget;
+#endif
   if (widget == NULL)
     /* State is defunct */
     return -1;
@@ -652,7 +706,11 @@ gucharmap_chartable_accessible_get_row_at_index (AtkTable *table,
   GucharmapChartable *chartable;
   GucharmapChartablePrivate *chartable_priv;
 
+#if GTK_CHECK_VERSION (2, 21, 0)
+  widget = gtk_accessible_get_widget (GTK_ACCESSIBLE (table));
+#else
   widget = GTK_ACCESSIBLE (table)->widget;
+#endif
   if (widget == NULL)
     /* State is defunct */
     return -1;
